# Estimate bridge reliability using Monte Carlo simulation and variance 
# reduction techniques
bridge=function(p)
{
  u=runif(5,0,1)
  s=as.numeric(u<=p)
  return(max(s[1]*s[4], s[1]*s[3]*s[5], s[2]*s[3]*s[4], s[2]*s[5]))
}
#
#Antithetic Variables
bridge_antithetic=function(p)
{
  u=runif(5,0,1)
  s=as.numeric(u<=p)
  x1=max(s[1]*s[4], s[1]*s[3]*s[5], s[2]*s[3]*s[4], s[2]*s[5])
  s=as.numeric(1-u<=p)
  x2=max(s[1]*s[4], s[1]*s[3]*s[5], s[2]*s[3]*s[4], s[2]*s[5])
  return(c(x1,x2))
}
# Control variate Y=number of working comonents Y=sum(S_i)
bridge_control_variate=function(p,n)
{
  # Create n observations from variable 
  # Z=X+c(Y-m)
  # where X=f(S_1,...,S_5) = indicator of working system
  # Y=sum(S_i) = number of working components
  # m=E(Y)=sum(p_i)
  # c=-cov(X,Y)/var(Y) (estimates from the n observations)
  # Estimate of theta=E(X)=E(Z) is given by mean(Z)
  #
  xy=matrix(rep(0,2*n), n,2)
  for (j in 1:n)
  {
    u=runif(5,0,1)
    s=as.numeric(u<=p)
    x=max(s[1]*s[4], s[1]*s[3]*s[5], s[2]*s[3]*s[4], s[2]*s[5])
    y=sum(s)
    xy[j,]=c(x,y)
  }
  c=-cov(xy[,1], xy[,2])/var(xy[,2])
  my=sum(p)
  z=xy[,1]+c*(xy[,2]-my)
  theta=mean(z)
  sd=sd(z)
  return(c(theta,sd))
}
# Apply the methods
# Straightforward Monte Carlo
p=c(1/3, 2/3, 3/4, 1/6, 4/5)
#
n=500
res1=rep(0,n)
for (j in 1:n)
{
  res1[j]=bridge(p)
}
theta1=mean(res1)
theta1
sd1=sd(res1)
sd1
#
# Antithetic Variables
n=250
res2=rep(0,2*n)
for (j in 1:n)
{
  x=bridge_antithetic(p)
  res2[2*j-1]=x[1]
  res2[2*j]=x[2]
}
res2
theta2=mean(res2)
sd2=sd(res2)
theta2
sd2
#
# Control Variate
n=500
res3=bridge_control_variate(p,n)
theta3=res3[1]
sd3=res3[2]
theta3
sd3
